'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _redux = require('redux');

var _reactRedux = require('react-redux');

var _Dialog = require('material-ui/Dialog');

var _Dialog2 = _interopRequireDefault(_Dialog);

var _FlatButton = require('material-ui/FlatButton');

var _FlatButton2 = _interopRequireDefault(_FlatButton);

var _Checkbox = require('material-ui/Checkbox');

var _Checkbox2 = _interopRequireDefault(_Checkbox);

var _TextField = require('material-ui/TextField');

var _TextField2 = _interopRequireDefault(_TextField);

var _actions = require('../actions');

var _styles = require('../styles');

var _styles2 = _interopRequireDefault(_styles);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let Settings = class Settings extends _react.Component {

  constructor(props) {
    super(props);

    this.handleLocalChecked = (e, checked) => {
      this.setState({ isCustom: checked });
    };

    this.handleInputChange = (e, value) => {
      this.options[e.target.id] = value;
    };

    this.handleCheckboxChange = (e, checked) => {
      this.options[e.target.id] = checked;
    };

    this.save = () => {
      this.props.saveSettings(this.state.isCustom, this.options);
      this.props.close();
    };

    const isCustom = props.isCustom;
    this.options = {};
    if (isCustom) {
      this.options.hostname = props.socketOptions.hostname;
      this.options.port = props.socketOptions.port;
      this.options.secure = props.socketOptions.secure;
    } else {
      this.options.hostname = 'localhost';
      this.options.port = '8000';
      this.options.secure = false;
    }

    this.state = { isCustom };
  }

  render() {
    const { isOpen, close } = this.props;

    const actions = [_react2.default.createElement(_FlatButton2.default, {
      label: 'Cancel',
      primary: true,
      onTouchTap: close
    }), _react2.default.createElement(_FlatButton2.default, {
      label: 'Submit',
      primary: true, keyboardFocused: true,
      onTouchTap: this.save
    })];

    return _react2.default.createElement(
      _Dialog2.default,
      {
        title: 'Settings',
        titleStyle: _styles2.default.dialogTitle,
        bodyStyle: _styles2.default.dialogBody,
        actions: actions,
        open: isOpen,
        onRequestClose: close,
        autoScrollBodyContent: true
      },
      _react2.default.createElement(_Checkbox2.default, {
        label: 'Use custom (local) server',
        checked: this.state.isCustom,
        onCheck: this.handleLocalChecked
      }),
      this.state.isCustom && _react2.default.createElement(
        'div',
        null,
        _react2.default.createElement(_TextField2.default, {
          id: 'hostname',
          floatingLabelText: 'Host name',
          defaultValue: this.options.hostname,
          onChange: this.handleInputChange
        }),
        _react2.default.createElement(_TextField2.default, {
          id: 'port',
          floatingLabelText: 'Port',
          defaultValue: this.options.port,
          onChange: this.handleInputChange
        }),
        _react2.default.createElement('br', null),
        _react2.default.createElement(_Checkbox2.default, {
          id: 'secure',
          label: 'Use secure connection',
          defaultChecked: this.options.secure,
          onCheck: this.handleCheckboxChange
        })
      )
    );
  }
};
Settings.propTypes = {
  isOpen: _react.PropTypes.bool,
  close: _react.PropTypes.func.isRequired,
  saveSettings: _react.PropTypes.func.isRequired,
  socketOptions: _react.PropTypes.object.isRequired,
  isCustom: _react.PropTypes.bool
};


function mapStateToProps(state) {
  return {
    socketOptions: state.socket.options,
    isCustom: state.socket.isCustom
  };
}

function mapDispatchToProps(dispatch) {
  return {
    saveSettings: (0, _redux.bindActionCreators)(_actions.saveSocketSettings, dispatch)
  };
}

exports.default = (0, _reactRedux.connect)(mapStateToProps, mapDispatchToProps)(Settings);
module.exports = exports['default'];