'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; // Based on https://github.com/YoruNoHikage/redux-devtools-dispatch

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reduxDevtoolsThemes = require('redux-devtools-themes');

var themes = _interopRequireWildcard(_reduxDevtoolsThemes);

var _redux = require('redux');

var _reactRedux = require('react-redux');

var _actions = require('../../actions');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const styles = {
  button: {
    cursor: 'pointer',
    fontWeight: 'bold',
    borderRadius: '3px',
    padding: '3px',
    margin: '5px',
    fontSize: '0.8em',
    textDecoration: 'none',
    border: 'none'
  },
  content: {
    margin: '5px',
    padding: '5px',
    borderRadius: '3px',
    outline: 'none',
    flex: '1 1 80%',
    overflow: 'auto'
  },
  label: {
    margin: '5px',
    padding: '5px',
    flex: '1 1 20%',
    overflow: 'hidden',
    textOverflow: 'ellipsis',
    direction: 'rtl',
    textAlign: 'left'
  }
};

let Dispatcher = class Dispatcher extends _react.Component {

  constructor(props, context) {
    super(props, context);
    this.state = {
      selected: 'default',
      args: []
    };
  }

  selectActionCreator(e) {
    const selected = e.target.value;
    let args = [];
    if (selected !== 'default') {
      // Shrink the number args to the number of the new ones
      args = this.state.args.slice(0, this.props.options.actionCreators[selected].args.length);
    }
    this.setState({
      selected,
      args
    });
  }

  handleArg(e, argIndex) {
    let value = this.refs['arg' + argIndex].textContent.trim();
    if (value === '') value = undefined;
    const args = [...this.state.args.slice(0, argIndex), value, ...this.state.args.slice(argIndex + 1)];
    this.setState({ args });
  }

  launchAction() {
    if (this.state.selected !== 'default') {
      let rest = this.refs.restArgs.textContent.trim();
      if (rest === '') rest = undefined;
      const { selected, args } = this.state;
      this.props.dispatch({
        name: this.props.options.actionCreators[selected].name,
        selected, args, rest
      });
    } else {
      if (this.refs.action.textContent !== '') {
        this.props.dispatch(this.refs.action.textContent);
      }
    }
  }

  componentDidMount() {
    this.resetCustomAction();
  }

  componentDidUpdate(prevProps, prevState) {
    if (this.state.selected === 'default' && prevState.selected !== 'default') {
      this.resetCustomAction();
    }
  }

  componentWillReceiveProps(nextProps) {
    if (this.state.selected !== 'default' && !nextProps.options.actionCreators) {
      this.setState({
        selected: 'default',
        args: []
      });
    }
  }

  resetCustomAction() {
    this.refs.action.innerHTML = this.props.options.lib === 'redux' ? '{<br/>type: \'\'<br/>}' : 'this.';
  }

  getTheme() {
    let { theme } = this.props;
    if (typeof theme !== 'string') {
      return theme;
    }

    if (typeof themes[theme] !== 'undefined') {
      return themes[theme];
    }

    console.warn('DevTools theme ' + theme + ' not found, defaulting to nicinabox');
    return themes.nicinabox;
  }

  render() {
    const theme = this.getTheme();
    const contentEditableStyle = _extends({}, styles.content, { color: theme.base06, backgroundColor: theme.base00
    });
    const buttonStyle = _extends({}, styles.button, { color: theme.base06, backgroundColor: theme.base00
    });
    const actionCreators = this.props.options.actionCreators;

    let fields = _react2.default.createElement('div', { contentEditable: true, style: contentEditableStyle, ref: 'action' });
    if (this.state.selected !== 'default' && actionCreators) {
      const fieldStyles = _extends({}, styles.label, { color: theme.base06 });
      fields = actionCreators[this.state.selected].args.map((param, i) => _react2.default.createElement(
        'div',
        { key: i, style: { display: 'flex' } },
        _react2.default.createElement(
          'span',
          { style: fieldStyles },
          param
        ),
        _react2.default.createElement('div', {
          contentEditable: true, style: contentEditableStyle, ref: 'arg' + i,
          onInput: e => this.handleArg(e, i)
        })
      ));
      fields.push(_react2.default.createElement(
        'div',
        { key: 'action', style: { display: 'flex' } },
        _react2.default.createElement(
          'span',
          { style: fieldStyles },
          'args...'
        ),
        _react2.default.createElement('div', { contentEditable: true, style: contentEditableStyle, ref: 'restArgs' })
      ));
    }

    let dispatchButtonStyle = buttonStyle;
    if (!actionCreators || actionCreators.length <= 0) {
      dispatchButtonStyle = _extends({}, buttonStyle, {
        position: 'absolute',
        bottom: '3px',
        right: '5px',
        background: theme.base02
      });
    }

    const dispatchButton = _react2.default.createElement(
      'button',
      { style: dispatchButtonStyle, onClick: this.launchAction.bind(this) },
      'Dispatch'
    );

    return _react2.default.createElement(
      'div',
      {
        style: {
          background: theme.base02,
          fontFamily: 'monaco,Consolas,Lucida Console,monospace',
          position: 'relative'
        }
      },
      fields,
      actionCreators && actionCreators.length > 0 ? _react2.default.createElement(
        'div',
        { style: { display: 'flex' } },
        _react2.default.createElement(
          'select',
          {
            onChange: this.selectActionCreator.bind(this),
            style: { flex: '1', margin: '5px 0 0 5px', height: '1.5em' },
            defaultValue: this.state.selected || 'default'
          },
          _react2.default.createElement(
            'option',
            { value: 'default' },
            'Custom action'
          ),
          actionCreators.map(({ name, func, args }, i) => _react2.default.createElement(
            'option',
            { key: i, value: i },
            name + '(' + args.join(', ') + ')'
          ))
        ),
        dispatchButton
      ) : dispatchButton
    );
  }
};
Dispatcher.propTypes = {
  options: _react.PropTypes.object.isRequired,
  dispatch: _react.PropTypes.func.isRequired,
  theme: _react.PropTypes.oneOfType([_react.PropTypes.object, _react.PropTypes.string])
};
Dispatcher.defaultProps = {
  theme: 'nicinabox'
};


function mapDispatchToProps(dispatch) {
  return {
    dispatch: (0, _redux.bindActionCreators)(_actions.dispatchRemotely, dispatch)
  };
}

exports.default = (0, _reactRedux.connect)(null, mapDispatchToProps)(Dispatcher);
module.exports = exports['default'];