"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = undefined;

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _errorStackParser = require("error-stack-parser");

var _errorStackParser2 = _interopRequireDefault(_errorStackParser);

var _getStackFrames = require("./react-error-overlay/utils/getStackFrames");

var _StackTrace = require("./react-error-overlay/containers/StackTrace");

var _StackTrace2 = _interopRequireDefault(_StackTrace);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let StackTraceTab = class StackTraceTab extends _react.Component {
    constructor(props) {
        super(props);

        this.onStackLocationClicked = (fileLocation = {}) => {
            //console.log("Stack location args: ", ...args);

            const parsedFramesNoSourcemaps = _errorStackParser2.default.parse(this.state.currentError);
            //console.log("Parsed stack frames: ", parsedFramesNoSourcemaps);

            const { fileName, lineNumber } = fileLocation;

            if (fileName && lineNumber) {
                const matchingStackFrame = this.state.stackFrames.find(stackFrame => {
                    const matches = stackFrame._originalFileName === fileName && stackFrame._originalLineNumber === lineNumber || stackFrame.fileName === fileName && stackFrame.lineNumber === lineNumber;
                    return matches;
                });

                //console.log("Matching stack frame: ", matchingStackFrame);

                if (matchingStackFrame) {
                    /*
                    const frameIndex = this.state.stackFrames.indexOf(matchingStackFrame);
                    const originalStackFrame = parsedFramesNoSourcemaps[frameIndex];
                    console.log("Original stack frame: ", originalStackFrame);
                    */
                    const adjustedLineNumber = Math.max(lineNumber - 1, 0);

                    chrome.devtools.panels.openResource(fileName, adjustedLineNumber, result => {
                        //console.log("openResource callback args: ", callbackArgs);
                        //console.log("Testing");
                        if (result.isError) {
                            const { fileName: finalFileName, lineNumber: finalLineNumber } = matchingStackFrame;
                            const adjustedLineNumber = Math.max(finalLineNumber - 1, 0);
                            chrome.devtools.panels.openResource(finalFileName, adjustedLineNumber, result => {
                                //console.log("openResource result: ", result);
                            });
                        }
                    });
                }
            }
        };

        this.state = {
            stackFrames: []
        };
    }
    componentDidMount() {
        //console.log("StackTraceTab mounted");
        this.checkForStackTrace();
    }

    componentDidUpdate(prevProps) {
        const { action, actions } = prevProps;

        if (action !== this.props.action || actions !== this.props.actions) {
            this.checkForStackTrace();
        }
    }

    checkForStackTrace() {
        const { action, actions: liftedActionsById } = this.props;

        if (!action) {
            return;
        }

        const liftedActions = Object.values(liftedActionsById);
        const liftedAction = liftedActions.find(liftedAction => liftedAction.action === action);

        if (liftedAction && typeof liftedAction.stack === 'string') {
            const deserializedError = Object.assign(new Error(), { stack: liftedAction.stack });

            (0, _getStackFrames.getStackFrames)(deserializedError).then(stackFrames => {
                console.log("Stack frames: ", stackFrames);
                this.setState({ stackFrames, currentError: deserializedError });
            });
        } else {
            this.setState({ stackFrames: [] });
        }
    }

    /*
    onStackFrameClicked = (i) => {
        const stackFrame = this.state.stackFrames[i];
         if(stackFrame) {
            const parsedFramesNoSourcemaps = ErrorStackParser.parse(this.state.currentError)
            console.log("Parsed stack frames: ", parsedFramesNoSourcemaps);
             if(chrome && chrome.devtools.panels.openResource) {
                const frameWithoutSourcemap = parsedFramesNoSourcemaps[i];
                const {fileName, lineNumber} = frameWithoutSourcemap;
                console.log("Parsed stack frame: ", stackFrame);
                console.log("Original stack frame: ", frameWithoutSourcemap);
                 const adjustedLineNumber = Math.max(lineNumber - 1, 0);
                  chrome.devtools.panels.openResource(fileName, adjustedLineNumber, (...callbackArgs) => {
                    console.log("openResource callback args: ", callbackArgs);
                    //console.log("Testing");
                });
            }
        }
    }
    */

    render() {
        const { stackFrames } = this.state;

        return _react2.default.createElement(
            "div",
            { style: { backgroundColor: "white", color: "black" } },
            _react2.default.createElement(
                "h2",
                null,
                "Dispatched Action Stack Trace"
            ),
            _react2.default.createElement(
                "div",
                { style: { display: "flex", flexDirection: "column" } },
                _react2.default.createElement(_StackTrace2.default, {
                    stackFrames: stackFrames,
                    errorName: "N/A",
                    contextSize: 3,
                    editorHandler: this.onStackLocationClicked
                })
            )
        );
    }
};
exports.default = StackTraceTab;
module.exports = exports["default"];