'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _StackFrameCodeBlock = require('./StackFrameCodeBlock');

var _StackFrameCodeBlock2 = _interopRequireDefault(_StackFrameCodeBlock);

var _getPrettyURL = require('../utils/getPrettyURL');

var _styles = require('../styles');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Copyright (c) 2015-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

const linkStyle = {
  fontSize: '0.9em',
  marginBottom: '0.9em'
};

const anchorStyle = {
  textDecoration: 'none',
  color: _styles.darkGray,
  cursor: 'pointer'
};

const codeAnchorStyle = {
  cursor: 'pointer'
};

const toggleStyle = {
  marginBottom: '1.5em',
  color: _styles.darkGray,
  cursor: 'pointer',
  border: 'none',
  display: 'block',
  width: '100%',
  textAlign: 'left',
  background: '#fff',
  fontFamily: 'Consolas, Menlo, monospace',
  fontSize: '1em',
  padding: '0px',
  lineHeight: '1.5'
};

let StackFrame = class StackFrame extends _react.Component {
  constructor(...args) {
    var _temp;

    return _temp = super(...args), this.state = {
      compiled: false
    }, this.toggleCompiled = () => {
      this.setState(state => ({
        compiled: !state.compiled
      }));
    }, this.editorHandler = () => {
      const errorLoc = this.getErrorLocation();
      if (!errorLoc) {
        return;
      }
      this.props.editorHandler(errorLoc);
    }, this.onKeyDown = e => {
      if (e.key === 'Enter') {
        this.editorHandler();
      }
    }, _temp;
  }

  getErrorLocation() {
    const {
      _originalFileName: fileName,
      _originalLineNumber: lineNumber
    } = this.props.frame;
    // Unknown file
    if (!fileName) {
      return null;
    }
    // e.g. "/path-to-my-app/webpack/bootstrap eaddeb46b67d75e4dfc1"
    const isInternalWebpackBootstrapCode = fileName.trim().indexOf(' ') !== -1;
    if (isInternalWebpackBootstrapCode) {
      return null;
    }
    // Code is in a real file
    return { fileName, lineNumber: lineNumber || 1 };
  }

  render() {
    const { frame, contextSize, critical, showCode } = this.props;
    const {
      fileName,
      lineNumber,
      columnNumber,
      _scriptCode: scriptLines,
      _originalFileName: sourceFileName,
      _originalLineNumber: sourceLineNumber,
      _originalColumnNumber: sourceColumnNumber,
      _originalScriptCode: sourceLines
    } = frame;
    const functionName = frame.getFunctionName();

    const compiled = this.state.compiled;
    const url = (0, _getPrettyURL.getPrettyURL)(sourceFileName, sourceLineNumber, sourceColumnNumber, fileName, lineNumber, columnNumber, compiled);

    let codeBlockProps = null;
    if (showCode) {
      if (compiled && scriptLines && scriptLines.length !== 0 && lineNumber != null) {
        codeBlockProps = {
          lines: scriptLines,
          lineNum: lineNumber,
          columnNum: columnNumber,
          contextSize,
          main: critical
        };
      } else if (!compiled && sourceLines && sourceLines.length !== 0 && sourceLineNumber != null) {
        codeBlockProps = {
          lines: sourceLines,
          lineNum: sourceLineNumber,
          columnNum: sourceColumnNumber,
          contextSize,
          main: critical
        };
      }
    }

    const canOpenInEditor = this.getErrorLocation() !== null && this.props.editorHandler !== null;
    return _react2.default.createElement(
      'div',
      null,
      _react2.default.createElement(
        'div',
        null,
        functionName
      ),
      _react2.default.createElement(
        'div',
        { style: linkStyle },
        _react2.default.createElement(
          'span',
          {
            style: canOpenInEditor ? anchorStyle : null,
            onClick: canOpenInEditor ? this.editorHandler : null,
            onKeyDown: canOpenInEditor ? this.onKeyDown : null,
            tabIndex: canOpenInEditor ? '0' : null
          },
          url
        )
      ),
      codeBlockProps && _react2.default.createElement(
        'span',
        null,
        _react2.default.createElement(
          'span',
          {
            onClick: canOpenInEditor ? this.editorHandler : null,
            style: canOpenInEditor ? codeAnchorStyle : null
          },
          _react2.default.createElement(_StackFrameCodeBlock2.default, codeBlockProps)
        ),
        _react2.default.createElement(
          'button',
          { style: toggleStyle, onClick: this.toggleCompiled },
          'View ' + (compiled ? 'source' : 'compiled')
        )
      )
    );
  }
};
exports.default = StackFrame;
module.exports = exports['default'];