'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _CodeBlock = require('../components/CodeBlock');

var _CodeBlock2 = _interopRequireDefault(_CodeBlock);

var _css = require('../utils/dom/css');

var _absolutifyCaret = require('../utils/dom/absolutifyCaret');

var _styles = require('../styles');

var _generateAnsiHTML = require('../utils/generateAnsiHTML');

var _generateAnsiHTML2 = _interopRequireDefault(_generateAnsiHTML);

var _codeFrame = require('@babel/code-frame');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Exact type workaround for spread operator.
// See: https://github.com/facebook/flow/issues/2405
function StackFrameCodeBlock(props) {
  const { lines, lineNum, columnNum, contextSize, main } = props;
  const sourceCode = [];
  let whiteSpace = Infinity;
  lines.forEach(function (e) {
    const { content: text } = e;
    const m = text.match(/^\s*/);
    if (text === '') {
      return;
    }
    if (m && m[0]) {
      whiteSpace = Math.min(whiteSpace, m[0].length);
    } else {
      whiteSpace = 0;
    }
  });
  lines.forEach(function (e) {
    let { content: text } = e;
    const { lineNumber: line } = e;

    if (isFinite(whiteSpace)) {
      text = text.substring(whiteSpace);
    }
    sourceCode[line - 1] = text;
  });
  const ansiHighlight = (0, _codeFrame.codeFrameColumns)(sourceCode.join('\n'), {
    start: {
      line: lineNum,
      column: columnNum == null ? 0 : columnNum - (isFinite(whiteSpace) ? whiteSpace : 0)
    }
  }, {
    forceColor: true,
    linesAbove: contextSize,
    linesBelow: contextSize
  });
  const htmlHighlight = (0, _generateAnsiHTML2.default)(ansiHighlight);
  const code = document.createElement('code');
  code.innerHTML = htmlHighlight;
  (0, _absolutifyCaret.absolutifyCaret)(code);

  const ccn = code.childNodes;
  // eslint-disable-next-line
  oLoop: for (let index = 0; index < ccn.length; ++index) {
    const node = ccn[index];
    const ccn2 = node.childNodes;
    for (let index2 = 0; index2 < ccn2.length; ++index2) {
      const lineNode = ccn2[index2];
      const text = lineNode.innerText;
      if (text == null) {
        continue;
      }
      if (text.indexOf(' ' + lineNum + ' |') === -1) {
        continue;
      }
      // $FlowFixMe
      (0, _css.applyStyles)(node, main ? _styles.primaryErrorStyle : _styles.secondaryErrorStyle);
      // eslint-disable-next-line
      break oLoop;
    }
  }

  return _react2.default.createElement(_CodeBlock2.default, { main: main, codeHTML: code.innerHTML });
} /**
   * Copyright (c) 2015-present, Facebook, Inc.
   *
   * This source code is licensed under the MIT license found in the
   * LICENSE file in the root directory of this source tree.
   */

exports.default = StackFrameCodeBlock;
module.exports = exports['default'];