'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _StackFrame = require('./StackFrame');

var _StackFrame2 = _interopRequireDefault(_StackFrame);

var _Collapsible = require('../components/Collapsible');

var _Collapsible2 = _interopRequireDefault(_Collapsible);

var _isInternalFile = require('../utils/isInternalFile');

var _isBultinErrorName = require('../utils/isBultinErrorName');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const traceStyle = {
  fontSize: '1em',
  flex: '0 1 auto',
  minHeight: '0px',
  overflow: 'auto'
}; /**
    * Copyright (c) 2015-present, Facebook, Inc.
    *
    * This source code is licensed under the MIT license found in the
    * LICENSE file in the root directory of this source tree.
    */

let StackTrace = class StackTrace extends _react.Component {
  renderFrames() {
    const { stackFrames, errorName, contextSize, editorHandler } = this.props;
    const renderedFrames = [];
    let hasReachedAppCode = false,
        currentBundle = [],
        bundleCount = 0;

    stackFrames.forEach((frame, index) => {
      const { fileName, _originalFileName: sourceFileName } = frame;
      const isInternalUrl = (0, _isInternalFile.isInternalFile)(sourceFileName, fileName);
      const isThrownIntentionally = !(0, _isBultinErrorName.isBultinErrorName)(errorName);
      const shouldCollapse = isInternalUrl && (isThrownIntentionally || hasReachedAppCode);

      if (!isInternalUrl) {
        hasReachedAppCode = true;
      }

      const frameEle = _react2.default.createElement(_StackFrame2.default, {
        key: 'frame-' + index,
        frame: frame,
        contextSize: contextSize,
        critical: index === 0,
        showCode: !shouldCollapse,
        editorHandler: editorHandler
      });
      const lastElement = index === stackFrames.length - 1;

      if (shouldCollapse) {
        currentBundle.push(frameEle);
      }

      if (!shouldCollapse || lastElement) {
        if (currentBundle.length === 1) {
          renderedFrames.push(currentBundle[0]);
        } else if (currentBundle.length > 1) {
          bundleCount++;
          renderedFrames.push(_react2.default.createElement(
            _Collapsible2.default,
            { key: 'bundle-' + bundleCount },
            currentBundle
          ));
        }
        currentBundle = [];
      }

      if (!shouldCollapse) {
        renderedFrames.push(frameEle);
      }
    });

    return renderedFrames;
  }

  render() {
    return _react2.default.createElement(
      'div',
      { style: traceStyle },
      this.renderFrames()
    );
  }
};
exports.default = StackTrace;
module.exports = exports['default'];