'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getStackFrames = undefined;

var _parser = require('./parser');

var _mapper = require('./mapper');

var _unmapper = require('./unmapper');

/**
 * Copyright (c) 2015-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

function getStackFrames(error, unhandledRejection = false, contextSize = 3) {
  const parsedFrames = (0, _parser.parse)(error);
  let enhancedFramesPromise;
  if (error.__unmap_source) {
    enhancedFramesPromise = (0, _unmapper.unmap)(
    // $FlowFixMe
    error.__unmap_source, parsedFrames, contextSize);
  } else {
    enhancedFramesPromise = (0, _mapper.map)(parsedFrames, contextSize);
  }
  return enhancedFramesPromise.then(enhancedFrames => {
    if (enhancedFrames.map(f => f._originalFileName).filter(f => f != null && f.indexOf('node_modules') === -1).length === 0) {
      return null;
    }
    return enhancedFrames.filter(({ functionName }) => functionName == null || functionName.indexOf('__stack_frame_overlay_proxy_console__') === -1);
  });
}

exports.default = getStackFrames;
exports.getStackFrames = getStackFrames;