'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.map = undefined;

var _stackFrame = require('./stack-frame');

var _stackFrame2 = _interopRequireDefault(_stackFrame);

var _getSourceMap = require('./getSourceMap');

var _getLinesAround = require('./getLinesAround');

var _settlePromise = require('settle-promise');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Enhances a set of <code>StackFrame</code>s with their original positions and code (when available).
 * @param {StackFrame[]} frames A set of <code>StackFrame</code>s which contain (generated) code positions.
 * @param {number} [contextLines=3] The number of lines to provide before and after the line specified in the <code>StackFrame</code>.
 */
/**
 * Copyright (c) 2015-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

async function map(frames, contextLines = 3) {
  const cache = {};
  const files = [];
  frames.forEach(frame => {
    const { fileName } = frame;
    if (fileName == null) {
      return;
    }
    if (files.indexOf(fileName) !== -1) {
      return;
    }
    files.push(fileName);
  });
  await (0, _settlePromise.settle)(files.map(async fileName => {
    const fileSource = await fetch(fileName).then(r => r.text());
    const map = await (0, _getSourceMap.getSourceMap)(fileName, fileSource);
    cache[fileName] = { fileSource, map };
  }));
  return frames.map(frame => {
    const { functionName, fileName, lineNumber, columnNumber } = frame;
    let { map, fileSource } = cache[fileName] || {};
    if (map == null || lineNumber == null) {
      return frame;
    }
    const { source, line, column } = map.getOriginalPosition(lineNumber, columnNumber);
    const originalSource = source == null ? [] : map.getSource(source) || [];
    return new _stackFrame2.default(functionName, fileName, lineNumber, columnNumber, (0, _getLinesAround.getLinesAround)(lineNumber, contextLines, fileSource), functionName, source, line, column, (0, _getLinesAround.getLinesAround)(line, contextLines, originalSource));
  });
}

exports.map = map;
exports.default = map;