'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getActiveInstance = exports.initialState = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.dispatchAction = dispatchAction;
exports.default = instances;

var _actionTypes = require('../constants/actionTypes');

var _socketActionTypes = require('../constants/socketActionTypes');

var _parseJSON = require('../utils/parseJSON');

var _parseJSON2 = _interopRequireDefault(_parseJSON);

var _updateState = require('../utils/updateState');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const initialState = exports.initialState = {
  selected: null,
  current: 'default',
  sync: false,
  connections: {},
  options: { default: {} },
  states: {
    default: {
      actionsById: {},
      computedStates: [],
      currentStateIndex: -1,
      nextActionId: 0,
      skippedActionIds: [],
      stagedActionIds: []
    }
  }
};

function updateState(state, request, id, serialize) {
  let payload = request.payload;
  const actionsById = request.actionsById;
  if (actionsById) {
    payload = _extends({}, payload, {
      actionsById: (0, _parseJSON2.default)(actionsById, serialize),
      computedStates: (0, _parseJSON2.default)(request.computedStates, serialize)
    });
    if (request.type === 'STATE' && request.committedState) {
      payload.committedState = payload.computedStates[0].state;
    }
  } else {
    payload = (0, _parseJSON2.default)(payload, serialize);
  }

  let newState;
  const liftedState = state[id] || state.default;
  const action = request.action && (0, _parseJSON2.default)(request.action) || {};

  switch (request.type) {
    case 'INIT':
      newState = (0, _updateState.recompute)(state.default, payload, { action: { type: '@@INIT' }, timestamp: action.timestamp || Date.now() });
      break;
    case 'ACTION':
      {
        let isExcess = request.isExcess;
        const nextActionId = request.nextActionId || liftedState.nextActionId + 1;
        if (typeof isExcess === 'undefined') isExcess = nextActionId > request.maxAge;
        if (Array.isArray(action)) {
          // Batched actions
          newState = liftedState;
          for (let i = 0; i < action.length; i++) {
            newState = (0, _updateState.recompute)(newState, request.batched ? payload : payload[i], action[i], newState.nextActionId + 1, isExcess);
          }
        } else {
          newState = (0, _updateState.recompute)(liftedState, payload, action, nextActionId, isExcess);
        }
        break;
      }
    case 'STATE':
      newState = payload;
      if (newState.computedStates.length <= newState.currentStateIndex) {
        newState.currentStateIndex = newState.computedStates.length - 1;
      }
      break;
    case 'PARTIAL_STATE':
      const maxAge = request.maxAge;
      const nextActionId = payload.nextActionId;
      const stagedActionIds = payload.stagedActionIds;
      let computedStates = payload.computedStates;
      let oldActionsById;
      let oldComputedStates;
      let committedState;
      if (nextActionId > maxAge) {
        const oldStagedActionIds = liftedState.stagedActionIds;
        const excess = oldStagedActionIds.indexOf(stagedActionIds[1]);
        let key;
        if (excess > 0) {
          oldComputedStates = liftedState.computedStates.slice(excess - 1);
          oldActionsById = _extends({}, liftedState.actionsById);
          for (let i = 1; i < excess; i++) {
            key = oldStagedActionIds[i];
            if (key) delete oldActionsById[key];
          }
          committedState = computedStates[0].state;
        } else {
          oldActionsById = liftedState.actionsById;
          oldComputedStates = liftedState.computedStates;
          committedState = liftedState.committedState;
        }
      } else {
        oldActionsById = liftedState.actionsById;
        oldComputedStates = liftedState.computedStates;
        committedState = liftedState.committedState;
      }
      computedStates = [...oldComputedStates, ...computedStates];
      const statesCount = computedStates.length;
      let currentStateIndex = payload.currentStateIndex;
      if (statesCount <= currentStateIndex) currentStateIndex = statesCount - 1;

      newState = _extends({}, liftedState, {
        actionsById: _extends({}, oldActionsById, payload.actionsById),
        computedStates,
        currentStateIndex,
        nextActionId,
        stagedActionIds,
        committedState
      });
      break;
    case 'LIFTED':
      newState = liftedState;
      break;
    default:
      return state;
  }

  if (request.liftedState) newState = _extends({}, newState, request.liftedState);
  return _extends({}, state, { [id]: newState });
}

function dispatchAction(state, { action }) {
  if (action.type === 'JUMP_TO_STATE' || action.type === 'JUMP_TO_ACTION') {
    const id = state.selected || state.current;
    const liftedState = state.states[id];
    let currentStateIndex = action.index;
    if (typeof currentStateIndex === 'undefined' && action.actionId) {
      currentStateIndex = liftedState.stagedActionIds.indexOf(action.actionId);
    }
    return _extends({}, state, {
      states: _extends({}, state.states, {
        [id]: _extends({}, liftedState, { currentStateIndex })
      })
    });
  }
  return state;
}

function removeState(state, connectionId) {
  const instanceIds = state.connections[connectionId];
  if (!instanceIds) return state;

  const connections = _extends({}, state.connections);
  const options = _extends({}, state.options);
  const states = _extends({}, state.states);
  let selected = state.selected;
  let current = state.current;
  let sync = state.sync;

  delete connections[connectionId];
  instanceIds.forEach(id => {
    if (id === selected) {
      selected = null;
      sync = false;
    }
    if (id === current) {
      const inst = Object.keys(connections)[0];
      if (inst) current = connections[inst][0];else current = 'default';
    }
    delete options[id];
    delete states[id];
  });
  return {
    selected,
    current,
    sync,
    connections,
    options,
    states
  };
}

function init({ type, action, name, libConfig = {} }, connectionId, current) {
  let lib;
  let actionCreators;
  let creators = libConfig.actionCreators || action;
  if (typeof creators === 'string') creators = JSON.parse(creators);
  if (Array.isArray(creators)) actionCreators = creators;
  if (type === 'STATE') lib = 'redux';
  return {
    name: libConfig.name || name || current,
    connectionId,
    explicitLib: libConfig.type,
    lib,
    actionCreators,
    features: libConfig.features ? libConfig.features : {
      lock: lib === 'redux', export: libConfig.type === 'redux' ? 'custom' : true,
      import: 'custom', persist: true, pause: true, reorder: true, jump: true, skip: true,
      dispatch: true, test: true
    },
    serialize: libConfig.serialize
  };
}

function instances(state = initialState, action) {
  switch (action.type) {
    case _actionTypes.UPDATE_STATE:
      const { request } = action;
      if (!request) return state;
      const connectionId = action.id || request.id;
      const current = request.instanceId || connectionId;
      let connections = state.connections;
      let options = state.options;

      if (typeof state.options[current] === 'undefined') {
        connections = _extends({}, state.connections, {
          [connectionId]: [...(connections[connectionId] || []), current]
        });
        options = _extends({}, options, { [current]: init(request, connectionId, current) });
      }

      return _extends({}, state, {
        current,
        connections,
        options,
        states: updateState(state.states, request, current, options[current].serialize)
      });
    case _actionTypes.SET_STATE:
      return _extends({}, state, {
        states: _extends({}, state.states, {
          [getActiveInstance(state)]: action.newState
        })
      });
    case _actionTypes.TOGGLE_SYNC:
      return _extends({}, state, { sync: !state.sync });
    case _actionTypes.SELECT_INSTANCE:
      return _extends({}, state, { selected: action.selected, sync: false });
    case _actionTypes.REMOVE_INSTANCE:
      return removeState(state, action.id);
    case _actionTypes.LIFTED_ACTION:
      {
        if (action.message === 'DISPATCH') return dispatchAction(state, action);
        if (action.message === 'IMPORT') {
          const id = state.selected || state.current;
          if (state.options[id].features.import === true) {
            return _extends({}, state, {
              states: _extends({}, state.states, {
                [id]: (0, _parseJSON2.default)(action.state)
              })
            });
          }
        }
        return state;
      }
    case _socketActionTypes.DISCONNECTED:
      return initialState;
    default:
      return state;
  }
}

/* eslint-disable no-shadow */
const getActiveInstance = exports.getActiveInstance = instances => instances.selected || instances.current;
/* eslint-enable */